!! This program contains an example of the 3d decomposition fftw
!! wrapper.  A typical program using this wrapper would follow a 
!! similar layout. 
!! If the -DCXMLCOMP flag is specified for the pre-processor,
!! then this example can be used to compare the results with
!! a single-precision random data set that was solved using
!! the cxml libraries using an alpha.

program cube
  implicit none
  include 'mpif.h'
  include 'fftw3d.fh'

#ifdef CXMLCOMP
  real(4), dimension(nc_in+2,nc_in,nc_in) :: input_data
  common / rrvar / input_data 
#endif
 
!! must be called to initialize communicators, etc

  call mpi_initialize

  call variable_initialize

!! be sure to set the number of openmp threads 
!! to match 'nt' found in the fftw3d.par parameter file.

  !$ call omp_set_num_threads(nt)

  if (rank == 0) write(*,*) 'starting main loop'

!! testing routine

  call test_fftw3d

!! send the fftw processes the signal to call mpi_finalize

  call fftw(0)
  call mpi_finalize(ierr)

contains
  
!! initialize variables
  subroutine variable_initialize

    implicit none

    ierr=-1
    slab=0.0
    cube_data=0.0

  end subroutine variable_initialize

!! initialize cube_data 
  subroutine initialize_data 
    implicit none
  
    integer(4) :: i,j,k,ii,jj,kk,i1

#ifdef CXMLCOMP
    open(unit=13,file='cxml_in.dat',form='binary')
    read(13) input_data
    close(13)
    i=1+nc_node_dim*cart_coords(3)
    j=1+nc_node_dim*cart_coords(2)
    k=1+nc_node_dim*cart_coords(1)
    ii=nc_node_dim*(cart_coords(3)+1)
    jj=nc_node_dim*(cart_coords(2)+1)
    kk=nc_node_dim*(cart_coords(1)+1)
    cube_data(:,:,:)=input_data(i:ii,j:jj,k:kk)
#else
    do j=0,cube_procs-1
      if (cube_rank.eq.j) then
        write(*,*) 'mr',cube_rank,'cart_coords',cart_coords
        do i=0,nodes_slab-1
          cube_data(:,:,1+nc_slab*i:nc_slab*(i+1))= &
                    cart_coords(1)*nodes_slab+i
        enddo
        write(*,*) 'mr',cube_rank,'sum cube initial=',sum(cube_data)
      endif
      call mpi_barrier(mpi_comm_cube,ierr)
    enddo
!      cube_data(:,:,:)=1.0
#endif

  end subroutine initialize_data

!! check complex data for correctness
  subroutine check_forward
    implicit none

    integer(4) :: i,j,k
    real(4) :: r,rr

    open(unit=13,file='cxml_cmplx.dat',form='binary')
    read(13) input_data
    close (13)
    do k=1,nc_slab
      do j=1,nc_dim
        do i=1,nc_dim+2
          rr=input_data(i,j,k+cube_rank*nc_slab)
          r=(slab(i,j,k)-rr)/slab(i,j,k)
          if (r.gt.0.01) then
            write(*,*) 'transformed mismatch:',i,j,k
            write(*,*) slab(i,j,k),rr,r*100,'%'
          endif
        enddo
      enddo
    enddo

  end subroutine

!! check real data for correctness
  subroutine check_back
    implicit none

    integer(4) :: i,j,k
    real(4) :: r,rr

    open(unit=13,file='cxml_final.dat',form='binary')
    read(13) input_data
    close (13)
    do k=1,nc_node_dim
      do j=1,nc_node_dim
        do i=1,nc_node_dim
          rr=input_data(i+nc_node_dim*cart_coords(3), &
                        j+nc_node_dim*cart_coords(2), &
                        k+nc_node_dim*cart_coords(1))
          r=(cube_data(i,j,k)-rr)/cube_data(i,j,k)
          if (r.gt.0.01) then
            write(*,*) 'real mismatch:',i,j,k
            write(*,*) cube_data(i,j,k),rr,r*100,'%'
          endif
        enddo
      enddo
    enddo

  end subroutine

!! fftw3d testing routine 
  subroutine test_fftw3d 
    implicit none

    integer(4) :: i,j,k

    call initialize_data

    if (rank == 0) write(*,*) 'finished initializing data'

    call fftw(1)

    if (rank == 0) write(*,*) 'finished forward fft'

#ifdef CXMLCOMP
    call check_forward
    if (rank == 0) write(*,*) 'finished forward check'
#endif


!! data is now in slab !!!

    call fftw(-1)

    if (rank == 0) write(*,*) 'finished backward fft'

!! data is now in cube !!!

#ifdef CXMLCOMP
    call check_back
#endif   
 
  end subroutine test_fftw3d

end program cube
