!***********************************************************
! fftw3d.f90 - cubic decomposition program for fftw

program fftw3d
  implicit none
  include 'mpif.h'
#ifdef IA64
  include 'fftw_f77.i'
#else
  include '/home/merz/include/fftw_f77.i'
#endif
  include 'fftw3d.par'

  integer(4), parameter :: order=FFTW_NORMAL_ORDER ! FFTW_TRANSPOSED_ORDER
  integer(4), parameter :: num_elements = (nc_dim+2)*nc_dim*nc_slab/nt

  integer(4) :: ierr, nn_returned, mr, calling_rank, tag
  integer(4) :: fftw_comm, fftw_rank, fftw_procs, subset, key
  integer(4), dimension(MPI_STATUS_SIZE) :: status
  integer(4) :: request, command
  logical :: comp_flag

#ifdef IA64
  integer(8) :: plan, iplan
#else
  integer(4) :: plan, iplan
#endif

  real(4), dimension(nc_dim+2,nc_dim,nc_slab/nt) :: data,work

  call mpi_init(ierr)
  call mpi_comm_size(mpi_comm_world, nn_returned, ierr)
  call mpi_comm_rank(mpi_comm_world, mr, ierr)

  subset = 1
  key = subset
  call mpi_comm_split(mpi_comm_world,subset,key,fftw_comm,ierr)
  call mpi_comm_size(fftw_comm, fftw_procs, ierr)
  call mpi_comm_rank(fftw_comm, fftw_rank, ierr)

  call rfftw3d_f77_mpi_create_plan(plan,fftw_comm,nc_dim, &
            nc_dim,nc_dim, FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE)
  call rfftw3d_f77_mpi_create_plan(iplan,fftw_comm,nc_dim, &
            nc_dim,nc_dim, FFTW_COMPLEX_TO_REAL, FFTW_ESTIMATE)

  calling_rank = fftw_rank / nt 
  tag = mr

  do 
    call mpi_irecv(command,1,mpi_integer,calling_rank,tag, &
                   mpi_comm_world,request,ierr)
#ifdef STEST 
    do
      call system('sleep 1')
      call mpi_test(request,comp_flag,status,ierr)
      if (comp_flag) exit
    enddo
#else
    call mpi_wait(request,status,ierr)
#endif
    if (command == 0) exit
    call mpi_irecv(data,num_elements,mpi_real,calling_rank,tag, &
                   mpi_comm_world,request,ierr)
#ifdef STEST 
    do 
      call system('sleep 1')
      call mpi_test(request,comp_flag,status,ierr)
      if (comp_flag) exit
    enddo
#else
    call mpi_wait(request,status,ierr)
#endif
    if (command > 0) then
      call rfftwnd_f77_mpi(plan,1,data,work,1,order)
    else
      call rfftwnd_f77_mpi(iplan,1,data,work,1,order)
      data=data/real(nc_dim*nc_dim*nc_dim)
    endif
    call mpi_isend(data,num_elements,mpi_real,calling_rank,tag, &
                 mpi_comm_world,request,ierr)
#ifdef STEST 
    do 
      call system('sleep 1')
      call mpi_test(request,comp_flag,status,ierr)
      if (comp_flag) exit
    enddo
#else
    call mpi_wait(request,status,ierr)
#endif
  enddo

  call rfftwnd_f77_mpi_destroy_plan(iplan)
  call rfftwnd_f77_mpi_destroy_plan(plan)
  call mpi_finalize(ierr)

end program fftw3d 
